@@ -1,6 +1,8 @@
+use strum::{Display, EnumString};
+
 use crate::{layout::Alignment, text::Line};
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Default, Clone, Eq, PartialEq, Hash)]
 pub struct Title<'a> {
     pub content: Line<'a>,
     /// Defaults to Left if unset
@@ -10,7 +12,7 @@ pub struct Title<'a> {
     pub position: Option<Position>,
 }
 
-#[derive(Debug, Default, Clone, Copy, PartialEq, Eq, Hash)]
+#[derive(Debug, Default, Display, EnumString, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum Position {
     #[default]
     Top,
@@ -46,12 +48,22 @@ where
     }
 }
 
-impl<'a> Default for Title<'a> {
-    fn default() -> Self {
-        Self {
-            content: Line::from(""),
-            alignment: None,
-            position: None,
-        }
+#[cfg(test)]
+mod tests {
+    use strum::ParseError;
+
+    use super::*;
+
+    #[test]
+    fn position_tostring() {
+        assert_eq!(Position::Top.to_string(), "Top");
+        assert_eq!(Position::Bottom.to_string(), "Bottom");
+    }
+
+    #[test]
+    fn position_from_str() {
+        assert_eq!("Top".parse::<Position>(), Ok(Position::Top));
+        assert_eq!("Bottom".parse::<Position>(), Ok(Position::Bottom));
+        assert_eq!("".parse::<Position>(), Err(ParseError::VariantNotFound));
     }
 }
