@@ -28,7 +28,7 @@ use super::{Block, Widget};
 ///     .data(BarGroup::default().bars(&[Bar::default().value(10), Bar::default().value(20)]))
 ///     .max(4);
 /// ```
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct BarChart<'a> {
     /// Block to wrap the widget in
     block: Option<Block<'a>>,
@@ -53,6 +53,8 @@ pub struct BarChart<'a> {
     /// Value necessary for a bar to reach the maximum height (if no value is specified,
     /// the maximum value in the data is taken as reference)
     max: Option<u64>,
+    /// direction of the bars
+    direction: Direction,
 }
 
 impl<'a> Default for BarChart<'a> {
@@ -69,6 +71,7 @@ impl<'a> Default for BarChart<'a> {
             group_gap: 0,
             bar_set: symbols::bar::NINE_LEVELS,
             style: Style::default(),
+            direction: Direction::Vertical,
         }
     }
 }
@@ -87,7 +90,10 @@ impl<'a> BarChart<'a> {
     ///        .data(BarGroup::default().bars(&[Bar::default().value(10), Bar::default().value(20)]));
     /// ```
     pub fn data(mut self, data: impl Into<BarGroup<'a>>) -> BarChart<'a> {
-        self.data.push(data.into());
+        let group: BarGroup = data.into();
+        if !group.bars.is_empty() {
+            self.data.push(group);
+        }
         self
     }
 
@@ -101,6 +107,9 @@ impl<'a> BarChart<'a> {
         self
     }
 
+    /// Set the default style of the bar.
+    /// It is also possible to set individually the style of each Bar.
+    /// In this case the default style will be patched by the individual style
     pub fn bar_style(mut self, style: Style) -> BarChart<'a> {
         self.bar_style = style;
         self
@@ -121,11 +130,17 @@ impl<'a> BarChart<'a> {
         self
     }
 
+    /// Set the default value style of the bar.
+    /// It is also possible to set individually the value style of each Bar.
+    /// In this case the default value style will be patched by the individual value style
     pub fn value_style(mut self, style: Style) -> BarChart<'a> {
         self.value_style = style;
         self
     }
 
+    /// Set the default label style of the groups and bars.
+    /// It is also possible to set individually the label style of each Bar or Group.
+    /// In this case the default label style will be patched by the individual label style
     pub fn label_style(mut self, style: Style) -> BarChart<'a> {
         self.label_style = style;
         self
@@ -140,6 +155,12 @@ impl<'a> BarChart<'a> {
         self.style = style;
         self
     }
+
+    /// Set the direction of the bars
+    pub fn direction(mut self, direction: Direction) -> BarChart<'a> {
+        self.direction = direction;
+        self
+    }
 }
 
 impl<'a> BarChart<'a> {
@@ -193,7 +214,72 @@ impl<'a> BarChart<'a> {
         }
     }
 
-    fn render_bars(&self, buf: &mut Buffer, bars_area: Rect, max: u64) {
+    fn render_horizontal_bars(self, buf: &mut Buffer, bars_area: Rect, max: u64) {
+        // convert the bar values to ratatui::symbols::bar::Set
+        let groups: Vec<Vec<u16>> = self
+            .data
+            .iter()
+            .map(|group| {
+                group
+                    .bars
+                    .iter()
+                    .map(|bar| (bar.value * u64::from(bars_area.width) / max) as u16)
+                    .collect()
+            })
+            .collect();
+
+        // print all visible bars
+        let mut bar_y = bars_area.top();
+        for (group_data, mut group) in groups.into_iter().zip(self.data) {
+            let bars = std::mem::take(&mut group.bars);
+
+            for (bar_length, bar) in group_data.into_iter().zip(bars) {
+                let bar_style = self.bar_style.patch(bar.style);
+
+                for y in 0..self.bar_width {
+                    let bar_y = bar_y + y;
+                    for x in 0..bars_area.width {
+                        let symbol = if x < bar_length {
+                            self.bar_set.full
+                        } else {
+                            self.bar_set.empty
+                        };
+                        buf.get_mut(bars_area.left() + x, bar_y)
+                            .set_symbol(symbol)
+                            .set_style(bar_style);
+                    }
+                }
+
+                let bar_value_area = Rect {
+                    y: bar_y + (self.bar_width >> 1),
+                    ..bars_area
+                };
+                bar.render_value_with_different_styles(
+                    buf,
+                    bar_value_area,
+                    bar_length as usize,
+                    self.value_style,
+                    self.bar_style,
+                );
+
+                bar_y += self.bar_gap + self.bar_width;
+            }
+
+            // if group_gap is zero, then there is no place to print the group label
+            // check also if the group label is still inside the visible area
+            let label_y = bar_y - self.bar_gap;
+            if self.group_gap > 0 && label_y < bars_area.bottom() {
+                let label_rect = Rect {
+                    y: label_y,
+                    ..bars_area
+                };
+                group.render_label(buf, label_rect, self.label_style);
+                bar_y += self.group_gap;
+            }
+        }
+    }
+
+    fn render_vertical_bars(&self, buf: &mut Buffer, bars_area: Rect, max: u64) {
         // convert the bar values to ratatui::symbols::bar::Set
         let mut groups: Vec<Vec<u64>> = self
             .data
@@ -224,7 +310,7 @@ impl<'a> BarChart<'a> {
                         _ => self.bar_set.full,
                     };
 
-                    let bar_style = bar.style.patch(self.bar_style);
+                    let bar_style = self.bar_style.patch(bar.style);
 
                     for x in 0..self.bar_width {
                         buf.get_mut(bar_x + x, bars_area.top() + j)
@@ -261,23 +347,24 @@ impl<'a> BarChart<'a> {
         // print labels and values in one go
         let mut bar_x = area.left();
         let bar_y = area.bottom() - label_height - 1;
-        for group in self.data.into_iter() {
-            // print group labels under the bars or the previous labels
-            if let Some(mut label) = group.label {
-                label.patch_style(self.label_style);
-                let label_max_width = group.bars.len() as u16 * self.bar_width
-                    + (group.bars.len() as u16 - 1) * self.bar_gap;
-
-                buf.set_line(
-                    bar_x + (label_max_width.saturating_sub(label.width() as u16) >> 1),
-                    area.bottom() - 1,
-                    &label,
-                    label_max_width,
-                );
+        for mut group in self.data.into_iter() {
+            if group.bars.is_empty() {
+                continue;
             }
+            let bars = std::mem::take(&mut group.bars);
+            // print group labels under the bars or the previous labels
+            let label_max_width =
+                bars.len() as u16 * (self.bar_width + self.bar_gap) - self.bar_gap;
+            let group_area = Rect {
+                x: bar_x,
+                y: area.bottom() - 1,
+                width: label_max_width,
+                height: 1,
+            };
+            group.render_label(buf, group_area, self.label_style);
 
             // print the bar values and numbers
-            for bar in group.bars.into_iter() {
+            for bar in bars.into_iter() {
                 bar.render_label_and_value(
                     buf,
                     self.bar_width,
@@ -311,16 +398,23 @@ impl<'a> Widget for BarChart<'a> {
 
         let max = self.maximum_data_value();
 
-        // remove invisible groups and bars, since we don't need to print them
-        self.remove_invisible_groups_and_bars(area.width);
-
-        let bars_area = Rect {
-            height: area.height - label_height,
-            ..area
-        };
-        self.render_bars(buf, bars_area, max);
-
-        self.render_labels_and_values(area, buf, label_height);
+        match self.direction {
+            Direction::Horizontal => {
+                // remove invisible groups and bars, since we don't need to print them
+                self.remove_invisible_groups_and_bars(area.height);
+                self.render_horizontal_bars(buf, area, max);
+            }
+            Direction::Vertical => {
+                // remove invisible groups and bars, since we don't need to print them
+                self.remove_invisible_groups_and_bars(area.width);
+                let bars_area = Rect {
+                    height: area.height - label_height,
+                    ..area
+                };
+                self.render_vertical_bars(buf, bars_area, max);
+                self.render_labels_and_values(area, buf, label_height);
+            }
+        }
     }
 }
 
@@ -611,7 +705,7 @@ mod tests {
         let expected = Buffer::with_lines(vec![
             "█   █     █       █  ",
             "█ █ █   █ █ █   █ █ █",
-            " G1      G2      G3  ",
+            "G1      G2      G3   ",
         ]);
 
         assert_buffer_eq!(buffer, expected);
@@ -634,7 +728,7 @@ mod tests {
         let expected = Buffer::with_lines(vec![
             "█   █     █      ",
             "█ █ █   █ █ █   █",
-            " G1      G2     G",
+            "G1      G2      G",
         ]);
         assert_buffer_eq!(buffer, expected);
     }
@@ -656,7 +750,7 @@ mod tests {
         let expected = Buffer::with_lines(vec![
             "█   █     █     ",
             "█ █ █   █ █ █   ",
-            " G1      G2     ",
+            "G1      G2      ",
         ]);
         assert_buffer_eq!(buffer, expected);
     }
@@ -737,4 +831,202 @@ mod tests {
                 .add_modifier(Modifier::BOLD)
         )
     }
+
+    #[test]
+    fn test_empty_group() {
+        let chart = BarChart::default()
+            .data(BarGroup::default().label("invisible".into()))
+            .data(
+                BarGroup::default()
+                    .label("G".into())
+                    .bars(&[Bar::default().value(1), Bar::default().value(2)]),
+            );
+
+        let mut buffer = Buffer::empty(Rect::new(0, 0, 3, 3));
+        chart.render(buffer.area, &mut buffer);
+        let expected = Buffer::with_lines(vec!["  █", "█ █", "G  "]);
+        assert_buffer_eq!(buffer, expected);
+    }
+
+    fn build_test_barchart<'a>() -> BarChart<'a> {
+        BarChart::default()
+            .data(BarGroup::default().label("G1".into()).bars(&[
+                Bar::default().value(2),
+                Bar::default().value(3),
+                Bar::default().value(4),
+            ]))
+            .data(BarGroup::default().label("G2".into()).bars(&[
+                Bar::default().value(3),
+                Bar::default().value(4),
+                Bar::default().value(5),
+            ]))
+            .group_gap(1)
+            .direction(Direction::Horizontal)
+            .bar_gap(0)
+    }
+
+    #[test]
+    fn test_horizontal_bars() {
+        let chart: BarChart<'_> = build_test_barchart();
+
+        let mut buffer = Buffer::empty(Rect::new(0, 0, 5, 8));
+        chart.render(buffer.area, &mut buffer);
+        let expected = Buffer::with_lines(vec![
+            "2█   ",
+            "3██  ",
+            "4███ ",
+            "G1   ",
+            "3██  ",
+            "4███ ",
+            "5████",
+            "G2   ",
+        ]);
+
+        assert_buffer_eq!(buffer, expected);
+    }
+
+    #[test]
+    fn test_horizontal_bars_no_space_for_group_label() {
+        let chart: BarChart<'_> = build_test_barchart();
+
+        let mut buffer = Buffer::empty(Rect::new(0, 0, 5, 7));
+        chart.render(buffer.area, &mut buffer);
+        let expected = Buffer::with_lines(vec![
+            "2█   ",
+            "3██  ",
+            "4███ ",
+            "G1   ",
+            "3██  ",
+            "4███ ",
+            "5████",
+        ]);
+
+        assert_buffer_eq!(buffer, expected);
+    }
+
+    #[test]
+    fn test_horizontal_bars_no_space_for_all_bars() {
+        let chart: BarChart<'_> = build_test_barchart();
+
+        let mut buffer = Buffer::empty(Rect::new(0, 0, 5, 5));
+        chart.render(buffer.area, &mut buffer);
+        let expected = Buffer::with_lines(vec!["2█   ", "3██  ", "4███ ", "G1   ", "3██  "]);
+
+        assert_buffer_eq!(buffer, expected);
+    }
+
+    fn test_horizontal_bars_label_width_greater_than_bar(bar_color: Option<Color>) {
+        let mut bar = Bar::default()
+            .value(2)
+            .text_value("label".into())
+            .value_style(Style::default().red());
+
+        if let Some(color) = bar_color {
+            bar = bar.style(Style::default().fg(color));
+        }
+
+        let chart: BarChart<'_> = BarChart::default()
+            .data(BarGroup::default().bars(&[bar, Bar::default().value(5)]))
+            .direction(Direction::Horizontal)
+            .bar_style(Style::default().yellow())
+            .value_style(Style::default().italic())
+            .bar_gap(0);
+
+        let mut buffer = Buffer::empty(Rect::new(0, 0, 5, 2));
+        chart.render(buffer.area, &mut buffer);
+
+        let mut expected = Buffer::with_lines(vec!["label", "5████"]);
+
+        // first line has a yellow foreground. first cell contains italic "5"
+        expected.get_mut(0, 1).modifier.insert(Modifier::ITALIC);
+        for x in 0..5 {
+            expected.get_mut(x, 1).set_fg(Color::Yellow);
+        }
+
+        let expected_color = if let Some(color) = bar_color {
+            color
+        } else {
+            Color::Yellow
+        };
+
+        // second line contains the word "label". Since the bar value is 2,
+        // then the first 2 characters of "label" are italic red.
+        // the rest is white (using the Bar's style).
+        let cell = expected.get_mut(0, 0).set_fg(Color::Red);
+        cell.modifier.insert(Modifier::ITALIC);
+        let cell = expected.get_mut(1, 0).set_fg(Color::Red);
+        cell.modifier.insert(Modifier::ITALIC);
+        expected.get_mut(2, 0).set_fg(expected_color);
+        expected.get_mut(3, 0).set_fg(expected_color);
+        expected.get_mut(4, 0).set_fg(expected_color);
+
+        assert_buffer_eq!(buffer, expected);
+    }
+
+    #[test]
+    fn test_horizontal_bars_label_width_greater_than_bar_without_style() {
+        test_horizontal_bars_label_width_greater_than_bar(None);
+    }
+
+    #[test]
+    fn test_horizontal_bars_label_width_greater_than_bar_with_style() {
+        test_horizontal_bars_label_width_greater_than_bar(Some(Color::White))
+    }
+
+    #[test]
+    fn test_group_label_style() {
+        let chart: BarChart<'_> = BarChart::default()
+            .data(
+                BarGroup::default()
+                    .label(Span::from("G1").red().into())
+                    .bars(&[Bar::default().value(2)]),
+            )
+            .group_gap(1)
+            .direction(Direction::Horizontal)
+            .label_style(Style::default().bold().yellow());
+
+        let mut buffer = Buffer::empty(Rect::new(0, 0, 5, 2));
+        chart.render(buffer.area, &mut buffer);
+
+        // G1 should have the bold red style
+        // bold: because of BarChart::label_style
+        // red: is included with the label itself
+        let mut expected = Buffer::with_lines(vec!["2████", "G1   "]);
+        let cell = expected.get_mut(0, 1).set_fg(Color::Red);
+        cell.modifier.insert(Modifier::BOLD);
+        let cell = expected.get_mut(1, 1).set_fg(Color::Red);
+        cell.modifier.insert(Modifier::BOLD);
+
+        assert_buffer_eq!(buffer, expected);
+    }
+
+    #[test]
+    fn test_group_label_center() {
+        let chart: BarChart<'_> = BarChart::default().data(
+            BarGroup::default()
+                .label(Line::from(Span::from("G")).alignment(Alignment::Center))
+                .bars(&[Bar::default().value(2), Bar::default().value(5)]),
+        );
+
+        let mut buffer = Buffer::empty(Rect::new(0, 0, 3, 3));
+        chart.render(buffer.area, &mut buffer);
+
+        let expected = Buffer::with_lines(vec!["  █", "▆ █", " G "]);
+        assert_buffer_eq!(buffer, expected);
+    }
+
+    #[test]
+    fn test_group_label_right() {
+        let chart: BarChart<'_> = BarChart::default().data(
+            BarGroup::default()
+                .label(Line::from(Span::from("G")).alignment(Alignment::Right))
+                .bars(&[Bar::default().value(2), Bar::default().value(5)]),
+        );
+
+        let mut buffer = Buffer::empty(Rect::new(0, 0, 3, 3));
+        chart.render(buffer.area, &mut buffer);
+
+        let expected = Buffer::with_lines(vec!["  █", "▆ █", "  G"]);
+        assert_buffer_eq!(buffer, expected);
+    }
 }
