@@ -23,7 +23,7 @@ use crate::{buffer::Buffer, layout::Rect, widgets::Widget};
 ///
 /// For a more complete example how to utilize `Clear` to realize popups see
 /// the example `examples/popup.rs`
-#[derive(Debug, Clone)]
+#[derive(Debug, Default, Clone, Eq, PartialEq, Hash)]
 pub struct Clear;
 
 impl Widget for Clear {
@@ -35,3 +35,28 @@ impl Widget for Clear {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::assert_buffer_eq;
+
+    #[test]
+    fn render() {
+        let mut buf = Buffer::with_lines(vec!["xxxxxxxxxxxxxxx"; 7]);
+        let clear = Clear;
+        clear.render(Rect::new(1, 2, 3, 4), &mut buf);
+        assert_buffer_eq!(
+            buf,
+            Buffer::with_lines(vec![
+                "xxxxxxxxxxxxxxx",
+                "xxxxxxxxxxxxxxx",
+                "x   xxxxxxxxxxx",
+                "x   xxxxxxxxxxx",
+                "x   xxxxxxxxxxx",
+                "x   xxxxxxxxxxx",
+                "xxxxxxxxxxxxxxx",
+            ])
+        );
+    }
+}
