@@ -5,10 +5,10 @@ use crate::{
     layout::{Corner, Rect},
     style::{Style, Styled},
     text::Text,
-    widgets::{Block, StatefulWidget, Widget},
+    widgets::{Block, HighlightSpacing, StatefulWidget, Widget},
 };
 
-#[derive(Debug, Clone, Default)]
+#[derive(Debug, Default, Clone, Eq, PartialEq, Hash)]
 pub struct ListState {
     offset: usize,
     selected: Option<usize>,
@@ -45,7 +45,7 @@ impl ListState {
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct ListItem<'a> {
     content: Text<'a>,
     style: Style,
@@ -90,7 +90,7 @@ impl<'a> ListItem<'a> {
 ///     .highlight_style(Style::default().add_modifier(Modifier::ITALIC))
 ///     .highlight_symbol(">>");
 /// ```
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct List<'a> {
     block: Option<Block<'a>>,
     items: Vec<ListItem<'a>>,
@@ -103,6 +103,8 @@ pub struct List<'a> {
     highlight_symbol: Option<&'a str>,
     /// Whether to repeat the highlight symbol for each line of the selected item
     repeat_highlight_symbol: bool,
+    /// Decides when to allocate spacing for the selection symbol
+    highlight_spacing: HighlightSpacing,
 }
 
 impl<'a> List<'a> {
@@ -118,6 +120,7 @@ impl<'a> List<'a> {
             highlight_style: Style::default(),
             highlight_symbol: None,
             repeat_highlight_symbol: false,
+            highlight_spacing: HighlightSpacing::default(),
         }
     }
 
@@ -146,6 +149,14 @@ impl<'a> List<'a> {
         self
     }
 
+    /// Set when to show the highlight spacing
+    ///
+    /// See [`HighlightSpacing`] about which variant affects spacing in which way
+    pub fn highlight_spacing(mut self, value: HighlightSpacing) -> Self {
+        self.highlight_spacing = value;
+        self
+    }
+
     pub fn start_corner(mut self, corner: Corner) -> List<'a> {
         self.start_corner = corner;
         self
@@ -228,7 +239,7 @@ impl<'a> StatefulWidget for List<'a> {
         let blank_symbol = " ".repeat(highlight_symbol.width());
 
         let mut current_height = 0;
-        let has_selection = state.selected.is_some();
+        let selection_spacing = self.highlight_spacing.should_add(state.selected.is_some());
         for (i, item) in self
             .items
             .iter_mut()
@@ -263,7 +274,7 @@ impl<'a> StatefulWidget for List<'a> {
                 } else {
                     &blank_symbol
                 };
-                let (elem_x, max_element_width) = if has_selection {
+                let (elem_x, max_element_width) = if selection_spacing {
                     let (elem_x, _) = buf.set_stringn(
                         x,
                         y + j as u16,
@@ -774,6 +785,134 @@ mod tests {
         assert_buffer_eq!(buffer, expected);
     }
 
+    #[test]
+    fn test_list_highlight_spacing_default_whenselected() {
+        // when not selected
+        {
+            let items = list_items(vec!["Item 0", "Item 1", "Item 2"]);
+            let list = List::new(items).highlight_symbol(">>");
+            let mut state = ListState::default();
+
+            let buffer = render_stateful_widget(list, &mut state, 10, 5);
+
+            let expected = Buffer::with_lines(vec![
+                "Item 0    ",
+                "Item 1    ",
+                "Item 2    ",
+                "          ",
+                "          ",
+            ]);
+            assert_buffer_eq!(buffer, expected);
+        }
+
+        // when selected
+        {
+            let items = list_items(vec!["Item 0", "Item 1", "Item 2"]);
+            let list = List::new(items).highlight_symbol(">>");
+            let mut state = ListState::default();
+            state.select(Some(1));
+
+            let buffer = render_stateful_widget(list, &mut state, 10, 5);
+
+            let expected = Buffer::with_lines(vec![
+                "  Item 0  ",
+                ">>Item 1  ",
+                "  Item 2  ",
+                "          ",
+                "          ",
+            ]);
+            assert_buffer_eq!(buffer, expected);
+        }
+    }
+
+    #[test]
+    fn test_list_highlight_spacing_default_always() {
+        // when not selected
+        {
+            let items = list_items(vec!["Item 0", "Item 1", "Item 2"]);
+            let list = List::new(items)
+                .highlight_symbol(">>")
+                .highlight_spacing(HighlightSpacing::Always);
+            let mut state = ListState::default();
+
+            let buffer = render_stateful_widget(list, &mut state, 10, 5);
+
+            let expected = Buffer::with_lines(vec![
+                "  Item 0  ",
+                "  Item 1  ",
+                "  Item 2  ",
+                "          ",
+                "          ",
+            ]);
+            assert_buffer_eq!(buffer, expected);
+        }
+
+        // when selected
+        {
+            let items = list_items(vec!["Item 0", "Item 1", "Item 2"]);
+            let list = List::new(items)
+                .highlight_symbol(">>")
+                .highlight_spacing(HighlightSpacing::Always);
+            let mut state = ListState::default();
+            state.select(Some(1));
+
+            let buffer = render_stateful_widget(list, &mut state, 10, 5);
+
+            let expected = Buffer::with_lines(vec![
+                "  Item 0  ",
+                ">>Item 1  ",
+                "  Item 2  ",
+                "          ",
+                "          ",
+            ]);
+            assert_buffer_eq!(buffer, expected);
+        }
+    }
+
+    #[test]
+    fn test_list_highlight_spacing_default_never() {
+        // when not selected
+        {
+            let items = list_items(vec!["Item 0", "Item 1", "Item 2"]);
+            let list = List::new(items)
+                .highlight_symbol(">>")
+                .highlight_spacing(HighlightSpacing::Never);
+            let mut state = ListState::default();
+
+            let buffer = render_stateful_widget(list, &mut state, 10, 5);
+
+            let expected = Buffer::with_lines(vec![
+                "Item 0    ",
+                "Item 1    ",
+                "Item 2    ",
+                "          ",
+                "          ",
+            ]);
+            assert_buffer_eq!(buffer, expected);
+        }
+
+        // when selected
+        {
+            let items = list_items(vec!["Item 0", "Item 1", "Item 2"]);
+            let list = List::new(items)
+                .highlight_symbol(">>")
+                .highlight_spacing(HighlightSpacing::Never);
+            let mut state = ListState::default();
+            state.select(Some(1));
+
+            let buffer = render_stateful_widget(list, &mut state, 10, 5);
+
+            let expected = Buffer::with_lines(vec![
+                "Item 0    ",
+                "Item 1    ",
+                "Item 2    ",
+                "          ",
+                "          ",
+            ]);
+            assert_buffer_eq!(buffer, expected);
+        }
+    }
+
     #[test]
     fn test_list_repeat_highlight_symbol() {
         let items = list_items(vec!["Item 0\nLine 2", "Item 1", "Item 2"]);
