@@ -1,58 +1,15 @@
+use strum::{Display, EnumString};
+
 use super::StatefulWidget;
 use crate::{
     buffer::Buffer,
     layout::Rect,
     style::Style,
-    symbols::{block::FULL, line},
-};
-
-/// Scrollbar Set
-/// ```text
-/// <--▮------->
-/// ^  ^   ^   ^
-/// │  │   │   └ end
-/// │  │   └──── track
-/// │  └──────── thumb
-/// └─────────── begin
-/// ```
-#[derive(Debug, Clone)]
-pub struct Set {
-    pub track: &'static str,
-    pub thumb: &'static str,
-    pub begin: &'static str,
-    pub end: &'static str,
-}
-
-pub const DOUBLE_VERTICAL: Set = Set {
-    track: line::DOUBLE_VERTICAL,
-    thumb: FULL,
-    begin: "▲",
-    end: "▼",
-};
-
-pub const DOUBLE_HORIZONTAL: Set = Set {
-    track: line::DOUBLE_HORIZONTAL,
-    thumb: FULL,
-    begin: "◄",
-    end: "►",
-};
-
-pub const VERTICAL: Set = Set {
-    track: line::VERTICAL,
-    thumb: FULL,
-    begin: "↑",
-    end: "↓",
-};
-
-pub const HORIZONTAL: Set = Set {
-    track: line::HORIZONTAL,
-    thumb: FULL,
-    begin: "←",
-    end: "→",
+    symbols::scrollbar::{Set, DOUBLE_HORIZONTAL, DOUBLE_VERTICAL},
 };
 
 /// An enum representing the direction of scrolling in a Scrollbar widget.
-#[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
+#[derive(Debug, Default, Display, EnumString, Clone, Copy, Eq, PartialEq, Hash)]
 pub enum ScrollDirection {
     /// Forward scroll direction, usually corresponds to scrolling downwards or rightwards.
     #[default]
@@ -80,7 +37,7 @@ pub enum ScrollDirection {
 ///
 /// If you don't have multi-line content, you can leave the `viewport_content_length` set to the
 /// default of 0 and it'll use the track size as a `viewport_content_length`.
-#[derive(Clone, Copy, Debug, Default)]
+#[derive(Debug, Default, Clone, Copy, Eq, PartialEq, Hash)]
 pub struct ScrollbarState {
     // The current position within the scrollable content.
     position: u16,
@@ -146,7 +103,7 @@ impl ScrollbarState {
 }
 
 /// Scrollbar Orientation
-#[derive(Default, Debug, Clone)]
+#[derive(Debug, Default, Display, EnumString, Clone, Eq, PartialEq, Hash)]
 pub enum ScrollbarOrientation {
     #[default]
     VerticalRight,
@@ -155,9 +112,8 @@ pub enum ScrollbarOrientation {
     HorizontalTop,
 }
 
-/// Scrollbar widget for tui-rs library.
+/// A widget to display a scrollbar
 ///
-/// This widget can be used to display a scrollbar in a terminal user interface.
 /// The following components of the scrollbar are customizable in symbol and style.
 ///
 /// ```text
@@ -168,13 +124,13 @@ pub enum ScrollbarOrientation {
 /// │  └──────── thumb
 /// └─────────── begin
 /// ```
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct Scrollbar<'a> {
     orientation: ScrollbarOrientation,
     thumb_style: Style,
     thumb_symbol: &'a str,
     track_style: Style,
-    track_symbol: &'a str,
+    track_symbol: Option<&'a str>,
     begin_symbol: Option<&'a str>,
     begin_style: Style,
     end_symbol: Option<&'a str>,
@@ -187,7 +143,7 @@ impl<'a> Default for Scrollbar<'a> {
             orientation: ScrollbarOrientation::default(),
             thumb_symbol: DOUBLE_VERTICAL.thumb,
             thumb_style: Style::default(),
-            track_symbol: DOUBLE_VERTICAL.track,
+            track_symbol: Some(DOUBLE_VERTICAL.track),
             track_style: Style::default(),
             begin_symbol: Some(DOUBLE_VERTICAL.begin),
             begin_style: Style::default(),
@@ -233,7 +189,7 @@ impl<'a> Scrollbar<'a> {
     }
 
     /// Sets the symbol that represents the track of the scrollbar.
-    pub fn track_symbol(mut self, track_symbol: &'a str) -> Self {
+    pub fn track_symbol(mut self, track_symbol: Option<&'a str>) -> Self {
         self.track_symbol = track_symbol;
         self
     }
@@ -279,12 +235,13 @@ impl<'a> Scrollbar<'a> {
     /// └─────────── begin
     /// ```
     ///
-    /// Only sets begin_symbol and end_symbol if they already contain a value.
-    /// If begin_symbol and/or end_symbol were set to `None` explicitly, this function will respect
-    /// that choice.
+    /// Only sets begin_symbol, end_symbol and track_symbol if they already contain a value.
+    /// If they were set to `None` explicitly, this function will respect that choice.
     pub fn symbols(mut self, symbol: Set) -> Self {
-        self.track_symbol = symbol.track;
         self.thumb_symbol = symbol.thumb;
+        if self.track_symbol.is_some() {
+            self.track_symbol = Some(symbol.track);
+        }
         if self.begin_symbol.is_some() {
             self.begin_symbol = Some(symbol.begin);
         }
@@ -471,8 +428,10 @@ impl<'a> StatefulWidget for Scrollbar<'a> {
         for i in track_start..track_end {
             let (style, symbol) = if i >= thumb_start && i < thumb_end {
                 (self.thumb_style, self.thumb_symbol)
+            } else if let Some(track_symbol) = self.track_symbol {
+                (self.track_style, track_symbol)
             } else {
-                (self.track_style, self.track_symbol)
+                continue;
             };
 
             if self.is_vertical() {
@@ -501,8 +460,79 @@ impl<'a> StatefulWidget for Scrollbar<'a> {
 
 #[cfg(test)]
 mod tests {
+    use strum::ParseError;
+
     use super::*;
-    use crate::assert_buffer_eq;
+    use crate::{
+        assert_buffer_eq,
+        symbols::scrollbar::{HORIZONTAL, VERTICAL},
+    };
+
+    #[test]
+    fn scroll_direction_to_string() {
+        assert_eq!(ScrollDirection::Forward.to_string(), "Forward");
+        assert_eq!(ScrollDirection::Backward.to_string(), "Backward");
+    }
+
+    #[test]
+    fn scroll_direction_from_str() {
+        assert_eq!(
+            "Forward".parse::<ScrollDirection>(),
+            Ok(ScrollDirection::Forward)
+        );
+        assert_eq!(
+            "Backward".parse::<ScrollDirection>(),
+            Ok(ScrollDirection::Backward)
+        );
+        assert_eq!(
+            "".parse::<ScrollDirection>(),
+            Err(ParseError::VariantNotFound)
+        );
+    }
+
+    #[test]
+    fn scrollbar_orientation_to_string() {
+        assert_eq!(
+            ScrollbarOrientation::VerticalRight.to_string(),
+            "VerticalRight"
+        );
+        assert_eq!(
+            ScrollbarOrientation::VerticalLeft.to_string(),
+            "VerticalLeft"
+        );
+        assert_eq!(
+            ScrollbarOrientation::HorizontalBottom.to_string(),
+            "HorizontalBottom"
+        );
+        assert_eq!(
+            ScrollbarOrientation::HorizontalTop.to_string(),
+            "HorizontalTop"
+        );
+    }
+
+    #[test]
+    fn scrollbar_orientation_from_str() {
+        assert_eq!(
+            "VerticalRight".parse::<ScrollbarOrientation>(),
+            Ok(ScrollbarOrientation::VerticalRight)
+        );
+        assert_eq!(
+            "VerticalLeft".parse::<ScrollbarOrientation>(),
+            Ok(ScrollbarOrientation::VerticalLeft)
+        );
+        assert_eq!(
+            "HorizontalBottom".parse::<ScrollbarOrientation>(),
+            Ok(ScrollbarOrientation::HorizontalBottom)
+        );
+        assert_eq!(
+            "HorizontalTop".parse::<ScrollbarOrientation>(),
+            Ok(ScrollbarOrientation::HorizontalTop)
+        );
+        assert_eq!(
+            "".parse::<ScrollbarOrientation>(),
+            Err(ParseError::VariantNotFound)
+        );
+    }
 
     #[test]
     fn test_no_render_when_area_zero() {
@@ -878,4 +908,28 @@ mod tests {
             assert_buffer_eq!(buffer, Buffer::with_lines(expected.clone()));
         }
     }
+
+    #[test]
+    fn test_rendering_without_track_horizontal_bottom() {
+        for i in 0..=16 {
+            let mut buffer = Buffer::empty(Rect::new(0, 0, 8, 2));
+            let mut state = ScrollbarState::default().position(i).content_length(16);
+            Scrollbar::default()
+                .orientation(ScrollbarOrientation::HorizontalBottom)
+                .track_symbol(None)
+                .render(buffer.area, &mut buffer, &mut state);
+            let expected = if i <= 1 {
+                vec!["        ", "◄██    ►"]
+            } else if i <= 5 {
+                vec!["        ", "◄ ██   ►"]
+            } else if i <= 9 {
+                vec!["        ", "◄  ██  ►"]
+            } else if i <= 13 {
+                vec!["        ", "◄   ██ ►"]
+            } else {
+                vec!["        ", "◄    ██►"]
+            };
+            assert_buffer_eq!(buffer, Buffer::with_lines(expected.clone()));
+        }
+    }
 }
