@@ -1,8 +1,9 @@
+use strum::{Display, EnumString};
 use unicode_width::UnicodeWidthStr;
 
 use crate::{
     buffer::Buffer,
-    layout::{Constraint, Direction, Layout, Rect},
+    layout::{Alignment, Constraint, Direction, Layout, Rect, SegmentSize},
     style::{Style, Styled},
     text::Text,
     widgets::{Block, StatefulWidget, Widget},
@@ -32,7 +33,7 @@ use crate::{
 ///
 /// You can apply a [`Style`] on the entire [`Cell`] using [`Cell::style`] or rely on the styling
 /// capabilities of [`Text`].
-#[derive(Debug, Clone, PartialEq, Eq, Default)]
+#[derive(Debug, Default, Clone, Eq, PartialEq, Hash)]
 pub struct Cell<'a> {
     content: Text<'a>,
     style: Style,
@@ -99,7 +100,7 @@ impl<'a> Styled for Cell<'a> {
 /// ```
 ///
 /// By default, a row has a height of 1 but you can change this using [`Row::height`].
-#[derive(Debug, Clone, PartialEq, Eq, Default)]
+#[derive(Debug, Default, Clone, Eq, PartialEq, Hash)]
 pub struct Row<'a> {
     cells: Vec<Cell<'a>>,
     height: u16,
@@ -160,6 +161,39 @@ impl<'a> Styled for Row<'a> {
     }
 }
 
+/// This option allows the user to configure the "highlight symbol" column width spacing
+#[derive(Debug, Display, EnumString, PartialEq, Eq, Clone, Default, Hash)]
+pub enum HighlightSpacing {
+    /// Always add spacing for the selection symbol column
+    ///
+    /// With this variant, the column for the selection symbol will always be allocated, and so the
+    /// table will never change size, regardless of if a row is selected or not
+    Always,
+    /// Only add spacing for the selection symbol column if a row is selected
+    ///
+    /// With this variant, the column for the selection symbol will only be allocated if there is a
+    /// selection, causing the table to shift if selected / unselected
+    #[default]
+    WhenSelected,
+    /// Never add spacing to the selection symbol column, regardless of whether something is
+    /// selected or not
+    ///
+    /// This means that the highlight symbol will never be drawn
+    Never,
+}
+
+impl HighlightSpacing {
+    /// Determine if a selection should be done, based on variant
+    /// Input "selection_state" should be similar to `state.selected.is_some()`
+    pub fn should_add(&self, selection_state: bool) -> bool {
+        match self {
+            HighlightSpacing::Always => true,
+            HighlightSpacing::WhenSelected => selection_state,
+            HighlightSpacing::Never => false,
+        }
+    }
+}
+
 /// A widget to display data in formatted columns.
 ///
 /// It is a collection of [`Row`]s, themselves composed of [`Cell`]s:
@@ -211,7 +245,7 @@ impl<'a> Styled for Row<'a> {
 /// // ...and potentially show a symbol in front of the selection.
 /// .highlight_symbol(">>");
 /// ```
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Default, Clone, Eq, PartialEq, Hash)]
 pub struct Table<'a> {
     /// A block to wrap the widget in
     block: Option<Block<'a>>,
@@ -229,6 +263,8 @@ pub struct Table<'a> {
     header: Option<Row<'a>>,
     /// Data to display in each row
     rows: Vec<Row<'a>>,
+    /// Decides when to allocate spacing for the row selection
+    highlight_spacing: HighlightSpacing,
 }
 
 impl<'a> Table<'a> {
@@ -245,6 +281,7 @@ impl<'a> Table<'a> {
             highlight_symbol: None,
             header: None,
             rows: rows.into_iter().collect(),
+            highlight_spacing: HighlightSpacing::default(),
         }
     }
 
@@ -286,17 +323,24 @@ impl<'a> Table<'a> {
         self
     }
 
+    /// Set when to show the highlight spacing
+    ///
+    /// See [`HighlightSpacing`] about which variant affects spacing in which way
+    pub fn highlight_spacing(mut self, value: HighlightSpacing) -> Self {
+        self.highlight_spacing = value;
+        self
+    }
+
     pub fn column_spacing(mut self, spacing: u16) -> Self {
         self.column_spacing = spacing;
         self
     }
 
-    fn get_columns_widths(&self, max_width: u16, has_selection: bool) -> Vec<u16> {
+    /// Get all offsets and widths of all user specified columns
+    /// Returns (x, width)
+    fn get_columns_widths(&self, max_width: u16, selection_width: u16) -> Vec<(u16, u16)> {
         let mut constraints = Vec::with_capacity(self.widths.len() * 2 + 1);
-        if has_selection {
-            let highlight_symbol_width = self.highlight_symbol.map_or(0, |s| s.width() as u16);
-            constraints.push(Constraint::Length(highlight_symbol_width));
-        }
+        constraints.push(Constraint::Length(selection_width));
         for constraint in self.widths {
             constraints.push(*constraint);
             constraints.push(Constraint::Length(self.column_spacing));
@@ -307,18 +351,19 @@ impl<'a> Table<'a> {
         let chunks = Layout::default()
             .direction(Direction::Horizontal)
             .constraints(constraints)
-            .expand_to_fill(false)
+            .segment_size(SegmentSize::None)
             .split(Rect {
                 x: 0,
                 y: 0,
                 width: max_width,
                 height: 1,
             });
-        let mut chunks = &chunks[..];
-        if has_selection {
-            chunks = &chunks[1..];
-        }
-        chunks.iter().step_by(2).map(|c| c.width).collect()
+        chunks
+            .iter()
+            .skip(1)
+            .step_by(2)
+            .map(|c| (c.x, c.width))
+            .collect()
     }
 
     fn get_row_bounds(
@@ -372,7 +417,7 @@ impl<'a> Styled for Table<'a> {
     }
 }
 
-#[derive(Debug, Clone, Default)]
+#[derive(Debug, Default, Clone, Eq, PartialEq, Hash)]
 pub struct TableState {
     offset: usize,
     selected: Option<usize>,
@@ -426,10 +471,13 @@ impl<'a> StatefulWidget for Table<'a> {
             None => area,
         };
 
-        let has_selection = state.selected.is_some();
-        let columns_widths = self.get_columns_widths(table_area.width, has_selection);
+        let selection_width = if self.highlight_spacing.should_add(state.selected.is_some()) {
+            self.highlight_symbol.map_or(0, |s| s.width() as u16)
+        } else {
+            0
+        };
+        let columns_widths = self.get_columns_widths(table_area.width, selection_width);
         let highlight_symbol = self.highlight_symbol.unwrap_or("");
-        let blank_symbol = " ".repeat(highlight_symbol.width());
         let mut current_height = 0;
         let mut rows_height = table_area.height;
 
@@ -445,22 +493,18 @@ impl<'a> StatefulWidget for Table<'a> {
                 },
                 header.style,
             );
-            let mut col = table_area.left();
-            if has_selection {
-                col += (highlight_symbol.width() as u16).min(table_area.width);
-            }
-            for (width, cell) in columns_widths.iter().zip(header.cells.iter()) {
+            let inner_offset = table_area.left();
+            for ((x, width), cell) in columns_widths.iter().zip(header.cells.iter()) {
                 render_cell(
                     buf,
                     cell,
                     Rect {
-                        x: col,
+                        x: inner_offset + x,
                         y: table_area.top(),
                         width: *width,
                         height: max_header_height,
                     },
                 );
-                col += *width + self.column_spacing;
             }
             current_height += max_header_height;
             rows_height = rows_height.saturating_sub(max_header_height);
@@ -479,41 +523,39 @@ impl<'a> StatefulWidget for Table<'a> {
             .skip(state.offset)
             .take(end - start)
         {
-            let (row, col) = (table_area.top() + current_height, table_area.left());
+            let (row, inner_offset) = (table_area.top() + current_height, table_area.left());
             current_height += table_row.total_height();
             let table_row_area = Rect {
-                x: col,
+                x: inner_offset,
                 y: row,
                 width: table_area.width,
                 height: table_row.height,
             };
             buf.set_style(table_row_area, table_row.style);
             let is_selected = state.selected.map_or(false, |s| s == i);
-            let table_row_start_col = if has_selection {
-                let symbol = if is_selected {
-                    highlight_symbol
-                } else {
-                    &blank_symbol
-                };
-                let (col, _) =
-                    buf.set_stringn(col, row, symbol, table_area.width as usize, table_row.style);
-                col
-            } else {
-                col
+            if selection_width > 0 && is_selected {
+                // this should in normal cases be safe, because "get_columns_widths" allocates
+                // "highlight_symbol.width()" space but "get_columns_widths"
+                // currently does not bind it to max table.width()
+                buf.set_stringn(
+                    inner_offset,
+                    row,
+                    highlight_symbol,
+                    table_area.width as usize,
+                    table_row.style,
+                );
             };
-            let mut col = table_row_start_col;
-            for (width, cell) in columns_widths.iter().zip(table_row.cells.iter()) {
+            for ((x, width), cell) in columns_widths.iter().zip(table_row.cells.iter()) {
                 render_cell(
                     buf,
                     cell,
                     Rect {
-                        x: col,
+                        x: inner_offset + x,
                         y: row,
                         width: *width,
                         height: table_row.height,
                     },
                 );
-                col += *width + self.column_spacing;
             }
             if is_selected {
                 buf.set_style(table_row_area, self.highlight_style);
@@ -528,7 +570,14 @@ fn render_cell(buf: &mut Buffer, cell: &Cell, area: Rect) {
         if i as u16 >= area.height {
             break;
         }
-        buf.set_line(area.x, area.y + i as u16, line, area.width);
+
+        let x_offset = match line.alignment {
+            Some(Alignment::Center) => (area.width / 2).saturating_sub(line.width() as u16 / 2),
+            Some(Alignment::Right) => area.width.saturating_sub(line.width() as u16),
+            _ => 0,
+        };
+
+        buf.set_line(area.x + x_offset, area.y + i as u16, line, area.width);
     }
 }
 
@@ -544,13 +593,145 @@ mod tests {
     use std::vec;
 
     use super::*;
-    use crate::style::{Color, Modifier, Style, Stylize};
+    use crate::{
+        layout::Constraint::*,
+        style::{Color, Modifier, Style, Stylize},
+        text::Line,
+    };
     #[test]
     #[should_panic]
     fn table_invalid_percentages() {
         Table::new(vec![]).widths(&[Constraint::Percentage(110)]);
     }
 
+    // test how constraints interact with table column width allocation
+    mod table_column_widths {
+        use super::*;
+
+        /// Construct a a new table with the given constraints, available and selection widths and
+        /// tests that the widths match the expected list of (x, width) tuples.
+        #[track_caller]
+        fn test(
+            constraints: &[Constraint],
+            available_width: u16,
+            selection_width: u16,
+            expected: &[(u16, u16)],
+        ) {
+            let table = Table::new(vec![]).widths(constraints);
+
+            let widths = table.get_columns_widths(available_width, selection_width);
+            assert_eq!(widths, expected);
+        }
+
+        #[test]
+        fn length_constraint() {
+            // without selection, more than needed width
+            test(&[Length(4), Length(4)], 20, 0, &[(0, 4), (5, 4)]);
+
+            // with selection, more than needed width
+            test(&[Length(4), Length(4)], 20, 3, &[(3, 4), (8, 4)]);
+
+            // without selection, less than needed width
+            test(&[Length(4), Length(4)], 7, 0, &[(0, 4), (5, 2)]);
+
+            // with selection, less than needed width
+            test(&[Length(4), Length(4)], 7, 3, &[(3, 4), (7, 0)]);
+        }
+
+        #[test]
+        fn max_constraint() {
+            // without selection, more than needed width
+            test(&[Max(4), Max(4)], 20, 0, &[(0, 4), (5, 4)]);
+
+            // with selection, more than needed width
+            test(&[Max(4), Max(4)], 20, 3, &[(3, 4), (8, 4)]);
+
+            // without selection, less than needed width
+            test(&[Max(4), Max(4)], 7, 0, &[(0, 4), (5, 2)]);
+
+            // with selection, less than needed width
+            test(&[Max(4), Max(4)], 7, 3, &[(3, 3), (7, 0)]);
+        }
+
+        #[test]
+        fn min_constraint() {
+            // in its currently stage, the "Min" constraint does not grow to use the possible
+            // available length and enabling "expand_to_fill" will just stretch the last
+            // constraint and not split it with all available constraints
+
+            // without selection, more than needed width
+            test(&[Min(4), Min(4)], 20, 0, &[(0, 4), (5, 4)]);
+
+            // with selection, more than needed width
+            test(&[Min(4), Min(4)], 20, 3, &[(3, 4), (8, 4)]);
+
+            // without selection, less than needed width
+            // allocates no spacer
+            test(&[Min(4), Min(4)], 7, 0, &[(0, 4), (4, 3)]);
+
+            // with selection, less than needed width
+            // allocates no selection and no spacer
+            test(&[Min(4), Min(4)], 7, 3, &[(0, 4), (4, 3)]);
+        }
+
+        #[test]
+        fn percentage_constraint() {
+            // without selection, more than needed width
+            test(&[Percentage(30), Percentage(30)], 20, 0, &[(0, 6), (7, 6)]);
+
+            // with selection, more than needed width
+            test(&[Percentage(30), Percentage(30)], 20, 3, &[(3, 6), (10, 6)]);
+
+            // without selection, less than needed width
+            // rounds from positions: [0.0, 0.0, 2.1, 3.1, 5.2, 7.0]
+            test(&[Percentage(30), Percentage(30)], 7, 0, &[(0, 2), (3, 2)]);
+
+            // with selection, less than needed width
+            // rounds from positions: [0.0, 3.0, 5.1, 6.1, 7.0, 7.0]
+            test(&[Percentage(30), Percentage(30)], 7, 3, &[(3, 2), (6, 1)]);
+        }
+
+        #[test]
+        fn ratio_constraint() {
+            // without selection, more than needed width
+            // rounds from positions: [0.00, 0.00, 6.67, 7.67, 14.33]
+            test(&[Ratio(1, 3), Ratio(1, 3)], 20, 0, &[(0, 7), (8, 6)]);
+
+            // with selection, more than needed width
+            // rounds from positions: [0.00, 3.00, 10.67, 17.33, 20.00]
+            test(&[Ratio(1, 3), Ratio(1, 3)], 20, 3, &[(3, 7), (11, 6)]);
+
+            // without selection, less than needed width
+            // rounds from positions: [0.00, 2.33, 3.33, 5.66, 7.00]
+            test(&[Ratio(1, 3), Ratio(1, 3)], 7, 0, &[(0, 2), (3, 3)]);
+
+            // with selection, less than needed width
+            // rounds from positions: [0.00, 3.00, 5.33, 6.33, 7.00, 7.00]
+            test(&[Ratio(1, 3), Ratio(1, 3)], 7, 3, &[(3, 2), (6, 1)]);
+        }
+    }
+
+    #[test]
+    fn test_render_table_with_alignment() {
+        let mut buf = Buffer::empty(Rect::new(0, 0, 20, 3));
+        let table = Table::new(vec![
+            Row::new(vec![Line::from("Left").alignment(Alignment::Left)]),
+            Row::new(vec![Line::from("Center").alignment(Alignment::Center)]),
+            Row::new(vec![Line::from("Right").alignment(Alignment::Right)]),
+        ])
+        .widths(&[Percentage(100)]);
+
+        Widget::render(table, Rect::new(0, 0, 20, 3), &mut buf);
+
+        let expected = Buffer::with_lines(vec![
+            "Left                ",
+            "       Center       ",
+            "               Right",
+        ]);
+
+        assert_eq!(buf, expected);
+    }
+
     #[test]
     fn cell_can_be_stylized() {
         assert_eq!(
@@ -596,4 +777,34 @@ mod tests {
                 .remove_modifier(Modifier::CROSSED_OUT)
         )
     }
+
+    #[test]
+    fn highlight_spacing_to_string() {
+        assert_eq!(HighlightSpacing::Always.to_string(), "Always".to_string());
+        assert_eq!(
+            HighlightSpacing::WhenSelected.to_string(),
+            "WhenSelected".to_string()
+        );
+        assert_eq!(HighlightSpacing::Never.to_string(), "Never".to_string());
+    }
+
+    #[test]
+    fn highlight_spacing_from_str() {
+        assert_eq!(
+            "Always".parse::<HighlightSpacing>(),
+            Ok(HighlightSpacing::Always)
+        );
+        assert_eq!(
+            "WhenSelected".parse::<HighlightSpacing>(),
+            Ok(HighlightSpacing::WhenSelected)
+        );
+        assert_eq!(
+            "Never".parse::<HighlightSpacing>(),
+            Ok(HighlightSpacing::Never)
+        );
+        assert_eq!(
+            "".parse::<HighlightSpacing>(),
+            Err(strum::ParseError::VariantNotFound)
+        );
+    }
 }
